#/usr/bin/python
# Client code for Update Agent
# Copyright (c) 1999-2001 Red Hat, Inc.  Distributed under GPL.
#
# Author: Preston Brown <pbrown@redhat.com>
#         Adrian Likins <alikins@redhat.com>
#
"""utility functions for up2date"""

import re
import os
import rpm
import string
import md5
from config import Up2dateConfig
cfg = Up2dateConfig()
cfg.load()

def getPackageSearchPath():
    dir_list = []
    dir_list.append(cfg.readEntry("storageDir"))

    dir_string = cfg.readEntry("packageDir")
    if dir_string: 
    	dir_list = dir_list + string.split(dir_string, ':')
    return dir_list

def pkgToString(pkg):
    return "%s-%s-%s" % (pkg[0], pkg[1], pkg[2])

# compare two RPM packages
def comparePackages(pkgLabel1, pkgLabel2):
    version1 = pkgLabel1[1]
    release1 = pkgLabel1[2]
    epoch1 = pkgLabel1[3]
    version2 = pkgLabel2[1]
    release2 = pkgLabel2[2]
    epoch2 = pkgLabel2[3]

    if epoch1 == "":
        epoch1 = None
    else:
        epoch1 = "%s" % epoch1
        
    if epoch2 == "":
            epoch2 = None
    else:
        epoch2 = "%s" % epoch2
        
    return rpm.labelCompare((epoch1, version1, release1),
                            (epoch2, version2, release2))


def md5sum(fileName):
    hash = md5.new()
    
    try:
        f = open(fileName, "r")
    except:
        return ""

    fData = f.read()
    hash.update(fData)
    del fData
    f.close()
    
    hex = string.hexdigits
    md5res = ""
    for c in hash.digest():
        i = ord(c)
        md5res = md5res + hex[(i >> 4) & 0xF] + hex[i & 0xf]

    return md5res



# grumble... need this for the commandline "up2date foo" scenario
# not real fond of having to have this info client side, but I can
# be somewhat more userfriendly if I do...

# NOTE: The order of the stuff in compat is in newest->latest order
# assuming backwards compability
compat_arches_table = {
    # ARCH      : # CANON,      # COMPAT
    "noarch"    : [ "noarch",   "noarch" ],
    "i386"      : [ "i386",     "i386", "noarch" ],
    "i486"      : [ "i386",     "i486", "i386", "noarch" ],
    "i586"      : [ "i386",     "i586", "i486", "i386", "noarch" ],
    "i686"      : [ "i386",     "i686", "i586", "i486", "i386", "noarch" ],
    "alpha"     : [ "alpha",    "alpha", "noarch" ],
    "alphaev6"  : [ "alpha",    "alphaev6", "alpha", "noarch" ],
    "sparc"     : [ "sparc",    "sparc", "noarch" ],
    "sparc64"   : [ "sparc",    "sparc64", "sparc", "noarch" ],
    "ia64"      : [ "ia64",     "ia64", "noarch" ],
    "src"       : [ "SRPMS",    "src" ],
    }

# translate an arch into its basename platform name
def realArch(arch):
    # XXX: this should not be really happening
    if not arch in compat_arches_table.keys():
        print "Can not locate architecture in internal database\n"
        print "Looking for %s in %s\n" % (arch, repr(compat_arches_table))
        return None
    return compat_arches_table[arch][0]

# just past in sysArch since theres not really any point in recomputing it
# sysArch is the arch of the actual system
# oldArch is the arch of the currently installed package
# newArch is the arch of the package were considering installing
#
# return value is a score of the newArch relative to the
# (sysArch, oldArch) pair, with oldArch being given priority
# A return of -1 signals incompatibility
def scoreArch(sysArch, oldArch, newArch, prefer_sysArch = 1):
    sysCompatArches = compatArch(sysArch)
    oldCompatArches = compatArch(oldArch)

    # We need data to work with
    if None in [sysCompatArches, oldCompatArches]:
        # there are no compatible arches for this system. Uh oh.
        return -1

    # A few sanity checks
    if newArch not in sysCompatArches:
        # the arch isnt supported on this platform
        return -1
    
    # if we prefer affinity to the system architecture, then we always
    # score against the sysArch list.
    if prefer_sysArch:
        return sysCompatArches.index(newArch)

    # If we prefer affinity to the arch of the installed package, do some
    # more checks.
    if oldArch not in sysCompatArches:
        # the version of the installed package is not compatible!!!
        return -1
    # Give preference to matches against the oldCompatArches
    if newArch in oldCompatArches:
        return oldCompatArches.index(newArch)
    # Otherwise, handicap the match into sysCompatArches by the distance of
    # oldArch from sysCompatArches
    return sysCompatArches.index(newArch) + sysCompatArches.index(oldArch)


# Get a list of compatible cpus
def compatArch(arch):
    # XXX: this should not be really happening
    if not arch in compat_arches_table.keys():
        return None
    return compat_arches_table[arch][1:]



def getArch():
    s = os.uname()[4]
    return s


def getBaseArch():
    arch = getArch()
    baseArch = realArch(arch)
    return baseArch

# return a glob for your particular architecture.
def archGlob():
    if re.search("i.86", os.uname()[4]):
        return "i?86"
    elif re.search("sparc", os.uname()[4]):
        return "sparc*"
    else:
        return os.uname()[4]




# small class for header caching pretending to be a dict
# in theory we could eventually make this smart and stuff
# (ie, mru, etc)
#
# implement it this way for now so we can make a max header
# cache size
#
# Need a miniHeaderCache, which caches all headers
# but only: name,version,release,arch,summary,description,size
# Otherwise the size computations are slow as dirt
#
class HeaderCache:
    def __init__(self):
        # how many headers to cache in ram
        if cfg.readEntry("headerCacheSize"):
            self.cache_size = cfg.readEntry("headerCacheSize")
        else:
            self.cache_size = 30
        self.__cache = {}
        self.__cacheLite = {}

    def set_cache_size(self, number_of_headers):
        self.cache_size = number_of_headers

    def __liteCopy(self, header):
        tmp = {}
        tmp['name'] = header['name']
        tmp['version'] = header['version']
        tmp['release'] = header['release']
        tmp['arch'] = header['arch']
        tmp['summary'] = header['summary']
        tmp['description'] = header['description']
        tmp['size'] = header['size']
        return tmp

    def __setitem__(self,item,value):
        if len(self.__cache) <= self.cache_size:
            self.__cache[item] = value
            self.__cacheLite[item] = self.__liteCopy(value)
        else:
            # okay, this is about as stupid of a cache as you can get
            # but if we hit the max cache size, this is as good as
            # any mechanism for freeing up space in the cache. This
            # would be a good place to put some smarts
            bar = self.__cache.keys()
            del self.__cache[bar[self.cache_size-1]]
            self.__cache[item] = value
            self.__cacheLite[item] = self.__liteCopy(value)

    def __getitem__(self, item):
        return self.__cache[item]
    

    def getLite(self, item):
        return self.__cacheLite[item]

    def __len__(self):
        return len(self.__cache)

    def keys(self):
        return self.__cache.keys()

    def values(self):
        return self.__cache.keys()

    def has_key(self, item,lite=None):
#        print "\n########\nhas_key called\n###########\n"
#        print "item: %s" % item
#        print self.__cache.keys()
        
        if lite:
            return self.__cacheLite.has_key(item)
        else:
            return self.__cache.has_key(item)

    def __delitem__(self, item):
        del self.__cache[item]

    def printLite(self):
        print self.__cacheLite
